package Carp::Always::SHS;
use parent qw(Carp::Always::SyntaxHighlightSource);

our $VERSION = '0.03'; # VERSION

1;
# ABSTRACT: Alias for Carp::Always::SyntaxHighlightSource

__END__

=pod

=encoding UTF-8

=head1 NAME

Carp::Always::SHS - Alias for Carp::Always::SyntaxHighlightSource

=head1 VERSION

This document describes version 0.03 of Carp::Always::SHS (from Perl distribution Carp-Always-SyntaxHighlightSource), released on 2016-03-16.

=head1 HOMEPAGE

Please visit the project's homepage at L<https://metacpan.org/release/Carp-Always-SyntaxHighlightSource>.

=head1 SOURCE

Source repository is at L<https://github.com/sharyanto/perl-Carp-Always-SyntaxHighlightSource>.

=head1 BUGS

Please report any bugs or feature requests on the bugtracker website L<https://rt.cpan.org/Public/Dist/Display.html?Name=Carp-Always-SyntaxHighlightSource>

When submitting a bug or request, please include a test-file or a
patch to an existing test-file that illustrates the bug or desired
feature.

=head1 AUTHOR

perlancar <perlancar@cpan.org>

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2016 by perlancar@cpan.org.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut
